@@ -81,14 +81,14 @@ subsequent delay."
   :global t :group 'corfu)
 
 (cl-defmethod corfu--exhibit :after (&context (corfu-echo-mode (eql t)) &optional _auto)
-  (if-let (((not (minibufferp)))
-           (delay (if (consp corfu-echo-delay)
-                      (funcall (if corfu-echo--message #'cdr #'car)
-                               corfu-echo-delay)
-                    corfu-echo-delay))
-           (fun (corfu--metadata-get 'company-docsig))
-           (cand (and (>= corfu--index 0)
-                      (nth corfu--index corfu--candidates))))
+  (if-let* (((not (minibufferp)))
+            (delay (if (consp corfu-echo-delay)
+                       (funcall (if corfu-echo--message #'cdr #'car)
+                                corfu-echo-delay)
+                     corfu-echo-delay))
+            (fun (corfu--metadata-get 'company-docsig))
+            (cand (and (>= corfu--index 0)
+                       (nth corfu--index corfu--candidates))))
       (if (<= delay 0)
           (corfu-echo--show (funcall fun cand))
         (corfu-echo--cancel)
